#ifndef __Rasters_h
#define __Rasters_h

void AbstractError(void);
unsigned NearAvailPlanes(unsigned n);

class Raster1DAbstract
	{
public: short UsageCount;
	int Size1D;
	void *Data1D;
	char Planes;

	Raster1DAbstract(void) {UsageCount=Planes=Size1D=0;Data1D=NULL;};
	virtual ~Raster1DAbstract() 		{Erase1DStub();};

	virtual void Allocate1D(int NewSize1D)	{Erase1DStub();};
	virtual void Erase(void)		{Erase1DStub();};
	virtual void Erase1D(void)		{Erase1DStub();};
	void Erase1DStub(void);

	virtual unsigned long GetValue1D(int x) const {return(0);};
	virtual int GetPlanes(void) const {return(0);};
	virtual int GetSize(void)   const {return(Size1D);};
	void *GetRow(void)          {return(Data1D);};
	virtual void SetValue1D(int x, unsigned long NewValue) {AbstractError();};

/*This set of functions copy internal data in the proper format to the given buffer.*/
	void Get(Raster1DAbstract &R1);
	virtual void Get1Bit(void *Buffer1Bit);
	virtual void Get2Bit(void *Buffer2Bit);
	virtual void Get4Bit(void *Buffer4Bit);
	virtual void Get8Bit(void *Buffer8Bit);
	virtual void Get16Bit(void *Buffer16Bit);
	virtual void Get24Bit(void *Buffer24Bit);
	virtual void Get32Bit(void *Buffer32Bit);

/*This set of functions receives given data and stores them to the internal structure.*/
	void Set(Raster1DAbstract &R1);
	virtual void Set1Bit(void *Buffer1Bit) {AbstractError();}
	virtual void Set2Bit(void *Buffer2Bit) {AbstractError();}
	virtual void Set4Bit(void *Buffer4Bit) {AbstractError();}
	virtual void Set8Bit(void *Buffer8Bit) {AbstractError();}
	virtual void Set16Bit(void *Buffer16Bit);
	virtual void Set24Bit(void *Buffer24Bit) {AbstractError();}
	virtual void Set32Bit(void *Buffer32Bit) {AbstractError();}

/*Bitwise operation with raster buffer*/
	virtual void Peel1Bit(void *Buffer1Bit,char plane);
	virtual void Join1Bit(void *Buffer1Bit,char plane);
	virtual void Peel8Bit(void *Buffer8Bit,char plane8) {AbstractError();}
	virtual void Join8Bit(void *Buffer8Bit,char plane8);

	virtual Raster1DAbstract *GetPalette(void) {return(NULL);}
	virtual void SetPalette(Raster1DAbstract *NewPalette) {AbstractError();}
	friend Raster1DAbstract *CreateRaster1D(int SizeX, int Planes);
	};
Raster1DAbstract *CreateRaster1D(int SizeX, int Planes);

class Raster2DAbstract:virtual public Raster1DAbstract
	{
public: int Size2D;
	void **Data2D;

	Raster2DAbstract(void) {Size2D=0;Data2D=NULL;}
	virtual ~Raster2DAbstract() 			{Erase2DStub();}

	virtual void Allocate2D(int NewSize1D, int NewSize2D);
	virtual void Erase(void)			{Erase2DStub();}
	virtual void Erase2D(void)			{Erase2DStub();}

	long GetValue2D(int Offset1D,int Offset2D);
	void SetValue2D(int Offset1D,int Offset2D,long x);
	void *GetRow(int Offset2D)   {if(Data2D==NULL || Offset2D>=Size2D) return(NULL);return(Data2D[Offset2D]);}
	Raster1DAbstract *GetRowRaster(int Offset2D);

	void Erase2DStub(void);

	friend Raster2DAbstract *CreateRaster2D(int SizeX, int SizeY, int Planes);	
	};
Raster2DAbstract *CreateRaster2D(int SizeX, int SizeY, int Planes);

class Raster2DColorAbstract:virtual public Raster1DAbstract
	{
public: Raster1DAbstract *palette;

	virtual Raster1DAbstract *GetPalette(void) {return(palette);}
	virtual void SetPalette(Raster1DAbstract *NewPalette) {if(palette!=NULL) delete(palette); palette=NewPalette;}
	};

class Raster3DAbstract:virtual public Raster2DAbstract
	{
public: int Size3D;
	void ***Data3D;

	Raster3DAbstract(void) {Size3D=0;Data3D=NULL;}
	virtual ~Raster3DAbstract() 			{Erase3DStub();}

	virtual void Allocate3D(int NewSize1D, int NewSize2D, int NewSize3D);
	virtual void Erase(void)			{Erase3DStub();}
	virtual void Erase3D(void)			{Erase3DStub();}

	long GetValue3D(int Offset1D,int Offset2D,int Offset3D);
	void SetValue3D(int Offset1D,int Offset2D,int Offset3D,long x);
	void *GetRow(int Offset3D)   {if(Data3D==NULL || Offset3D>=Size3D) return(NULL);return(Data3D[Offset3D]);}
	Raster2DAbstract *GetRowRaster(int Offset3D);

	void Erase3DStub(void);

	friend Raster3DAbstract *CreateRaster3D(int SizeX, int SizeY, int SizeZ, int Planes);
	};
Raster3DAbstract *CreateRaster3D(int SizeX, int SizeY, int SizeZ, int Planes);


/* ---------- Global class Image --------- */

/*Definition of errors while Loading/Saving File(s)*/
#define ErrOK		  0
#define ErrOpenFile	 -1
#define ErrReadFile	 -2
#define ErrWriteFile	 -3
#define ErrBadFileFormat -4

#define ErrNoMem	-10



#ifndef RGBQuad_DEFINED
#define RGBQuad_DEFINED
struct RGBQuad
	{
	unsigned R,G,B,O;
	};
#endif

class APalette:virtual public Raster1DAbstract
	{
public:	unsigned R(unsigned index) {return(GetValue1D(3*index));}
	unsigned G(unsigned index) {return(GetValue1D(3*index+1));}
	unsigned B(unsigned index) {return(GetValue1D(3*index+2));}

	unsigned R(unsigned index,unsigned newR) {SetValue1D(3*index,newR);return(newR);};
	unsigned G(unsigned index,unsigned newG) {SetValue1D(3*index+1,newG);return(newG);};
	unsigned B(unsigned index,unsigned newB) {SetValue1D(3*index+2,newB);return(newB);};

        virtual void Get(unsigned index, RGBQuad *RGB);
	friend APalette *BuildPalette(int Indices, char type);
	};
APalette *BuildPalette(int Indices, char type=0);

typedef enum
  {
  ImageNone = 0,      //-none,
  ImageGray,          //1-gray,
  ImagePalette,       //2-palette,
  ImageTrueColor,     //3-true color
  ImagePaletteF = 0x80, //there is used abundant palette
  ImageReal = 0x100,  //Real numbers are used
  ImageSigned = 0x200 //image uses signed numbers
  } IMAGE_TYPE;

class Image
	{
public: Raster2DAbstract *Raster;
	APalette *Palette;
	Image *Next;
	float x,y,dx,dy,RotAngle;
	char *Comment;

	Image(void): Comment(NULL)  {RotAngle=x=y=dx=dy=0;Raster=NULL;Next=NULL;Palette=NULL;};
	Image(const Image & I);
	Image(Raster2DAbstract *NewRaster): Comment(NULL) {x=y=dx=dy=0;Next=NULL;Palette=NULL;if((Raster=NewRaster)!=NULL) Raster->UsageCount++;};
	~Image(void) {Erase();};
	IMAGE_TYPE ImageType(void) const;   // 0-none, 1-gray, 2-palette, 3-true color

	Raster2DAbstract *operator=(Raster2DAbstract *NewRaster);
	Image &operator=(const Image & I);

	unsigned long GetPixel(int SizeX, int SizeY) const {return(Raster==NULL?0:Raster->GetValue2D(SizeX,SizeY));};
	void SetPixel(int SizeX, int SizeY, unsigned long x) {if(Raster) Raster->SetValue2D(SizeX,SizeY,x);};

	void Create(int SizeX, int SizeY, int Planes);
	void Erase(void);
	};


void Flip1D(Raster2DAbstract *r2D);
void Flip2D(Raster2DAbstract *r2D);
int GrayPalette(Raster1DAbstract *Palette,int Planes=0);
int FillGray(Raster1DAbstract *Palette,int Planes);


/* -----File format loader dynamical list------ */
typedef Image (* TLoadPicture)(const char *Name);
typedef int (*TSavePicture)(const char *Name, const Image &Img);

class TImageFileHandler
	{
private:static TImageFileHandler *First;
	TImageFileHandler *Previous;
	TImageFileHandler *Next;

	const char *Extension;

public: TImageFileHandler(const char *NewShortKey, TLoadPicture LoadPicture_XXX = NULL, TSavePicture SavePicture_XXX=NULL);
	~TImageFileHandler();

	TLoadPicture LoadPicture;
	TSavePicture SavePicture;
	const char *extension() {return(Extension);}

	static TImageFileHandler *first() {return(First);}
	TImageFileHandler *next() {return(Next);}
	};

int SavePicture(const char *Name,const Image &Img);

#endif //__Rasters_h
